#!/usr/bin/env python2

import logging
import os
import json
import glob
import hashlib
from itertools import chain

import servicemanager

logger=logging.getLogger('pusher')

# --------------- Class to handle status of files ------------------
class status:
    ST_MODIFIED="M"
    ST_UNTRACKED="?"
    ST_UPTODATE="S"
    ST_ADDED="A"
    ST_DELETED="D"
    ST_COMPLICATED="C"

    """Managing status requests"""
    def __init__(self):
        self.DB_FILE=".pusherdb"
        self.sman=servicemanager.servicemanager()

    def GetStatus(self,directory,files=None,service=None):
        """ Scans through all files in direcotory,
        opens DB (if any), and returns dictionary (where filename is key)
        of files and their status

        - Returns dictionary where filename is key
        - Status can be:
            - ? : Not tracked
            - S : Synched, in DB and on FLICKR
            - A : Added, will be pushed on checkin
            - M : Modified locally, will push to flicker on checkin
            - D : Deleted locally, will remove from flickr on checkin
            - C : Complicated (different states depending on service)
        Dictionary has format
        d['service']    - Class object handling this file (None if not handled)
        d['status']     - ?/S/A/M/D/C
        - Hash never computed here
        
        Also returns DB dictionary, where filename is key"""
        logger.debug("GetStatus dir=%s" %(directory))

        db=self._loadDB(directory)

        # Get file stats
        filedict=self._findFiles(directory,files)

        logger.debug("Found %d files in dir"%(len(filedict.keys())))
        logger.debug("DB has %d files"%(len(db.keys())))

        # Now go through each file and see if in dictionary, compare mtimes
        keys=filedict.keys()
        keys.sort()
        for fn in keys:
            if fn in db:
                filedict[fn]['status']=self._computeStatus(db[fn],service)
                # Only Set the modify flag if the current
                # file was actually synched in the first place
                if filedict[fn]['mtime'] > db[fn]['mtime']:
                    # Compute a new hash
                    newhash=self._hashfile(os.path.join(directory,fn))
                    for srv in db[fn]['services']:
                        if db[fn]['services'][srv]['status']\
                                ==self.ST_UPTODATE:
                            # Only if the hash changed do we update
                            if newhash!=db[fn]['hash']:
                                filedict[fn]['status']=self.ST_MODIFIED
                                db[fn]['services'][srv]['status']\
                                        =self.ST_MODIFIED
                            else:
                                # Hash didn't change, let's update the mtime
                                # so we don't have to hash again
                                db[fn]['mtime']=filedict[fn]['mtime']
                                self._saveDB(directory,db)
                # Recompute since status might have changed
                filedict[fn]['status']=self._computeStatus(db[fn],service)
            else:
                filedict[fn]['status']=self.ST_UNTRACKED


        return filedict,db

    def _computeStatus(self, dfile, service):
        """Computes status for file, basically this means if 
        more than one service handles the file, it will place
        a 'C' (for complicated) otherwise if status matches
        between all services, will place that status"""

        # If only one service requested
        if service:
            if not dfile['services'].has_key(service):
                return self.ST_UNTRACKED
            else:
                return dfile['services'][service]['status']

        # Otherwise go through all services and compute
        # a sensible status
        first_service_key=dfile['services'].keys()[0]

        # Save off one of the statuses so we can compute
        # if they are all the same between services.
        first_status=dfile['services'][first_service_key]['status']

        all_status_match=True

        # Return ST_COMPLICATED "C" if status
        # differs
        for service in dfile['services']:
            if dfile['services'][service]['status']!=first_status:
                return self.ST_COMPLICATED

        return first_status

    def UpdateStatus(self,directory,files,status,service):
        """Given a list of files in a certain directory,
        updates to the given status
        If status is ST_UPTODATE, and no files given
        will go through current DB and push files
        with the "A" flag
        service: If none, assumes all services, looks for matching service
        """

        # First get a list of all files in dir
        # as well as current status
        dfiles,db=self.GetStatus(directory,files)

        # If no file list given and asked to 
        # bring files up to date, go through files in DB
        # with "A" or M status
        if not files and status==self.ST_UPTODATE:
            files=[]
            logger.debug('No files given, finding all that needs uploading')
            for fn in dfiles:
                if dfiles[fn]['status']==self.ST_ADDED\
                        or dfiles[fn]['status']==self.ST_MODIFIED\
                        or dfiles[fn]['status']==self.ST_COMPLICATED\
                        or dfiles[fn]['status']==self.ST_DELETED:
                    files.append(fn)
                    logger.debug('Adding %s to list with status %s'\
                            %(fn,dfiles[fn]['status']))

        # Go through list of files given, see if they
        # actually exist
        files_that_exist=[]
        for fn in files:
            if fn in dfiles:
                files_that_exist.append(fn)
            else:
                logger.debug("%s - Skipping, does not exist"%(fn))

        files_that_exist.sort()

        # FIXME: This is bad, but flickr needs to upload
        # media files first, then meta data files. Here
        # we ensure this happens
        if service is None or service is 'flickr':
            sobj=self.sman.GetServiceObj('flickr')
            files_hack_media=[]
            files_hack_config=[]
            files_hack_other=[]
            for filename in files_that_exist:
                if sobj._isMediaFile(filename):
                    files_hack_media.append(filename)
                elif sobj._isConfigFile(filename):
                    files_hack_config.append(filename)
                else:
                    files_hack_other.append(filename)


            #Now organize as we'd like it, media first, then config, then others
            files_that_exist=[]
            files_that_exist.append(files_hack_media)
            files_that_exist.append(files_hack_config)
            files_that_exist.append(files_hack_other)
            # Flatten list
            files_that_exist=list(chain.from_iterable(files_that_exist))

        for fn in files_that_exist:
            if status==self.ST_ADDED:
                self._updateToAdded(directory,fn,dfiles[fn],db,service)
            elif status==self.ST_UPTODATE:
                if not db.has_key(fn):
                    print("%s - Not in db, did you do an 'add' first?"%(fn))
                    continue
                if service:
                    servicelist=[service]
                else:
                    servicelist=db[fn]['services'].keys()

                for sn in servicelist:
                    if not db[fn]['services'].has_key(sn):
                        print("%s - Service [%s] unknown, skipping"\
                                %(fn,sn))
                        continue
                    # If push request and file is marked for
                    # deletion, delete it rather from remote
                    if db[fn]['services'][sn]['status']==self.ST_DELETED:
                        self._deleteFile(directory,fn,dfiles[fn],db,sn)
                    else:
                        self._uploadFile(directory,fn,dfiles[fn],db,sn)

            elif status==self.ST_DELETED:
                self._updateToDeleted(directory,fn,dfiles[fn],db,service)
            # Save for every upload, so we can easily resume
            self._saveDB(directory,db)

        # Always save when status is updated
        self._saveDB(directory,db)

        # Now print out updated status
        self.PrintStatus(directory,files)

    def _deleteFile(self,directory,fn,dentry,db,service):
        """Deletets file and changes status to '?' if no
        more services manages the file
        """
        # FIXME : can switch back to only managing once service
        # at a time
        logger.debug("%s - Deleting"%(fn))

        if fn not in db:
            print("%s - rm: Not in DB, can't remove !"%(fn))
            return False

        # Build up list of names
        servicenames=db[fn]['services'].keys()
        # If service is none, build list of all services
        # to perform this action on
        if service is None:
            servicelist=servicenames
        else:
            servicelist=[service]

        for service in servicelist:
            if not db[fn]['services'].has_key(service):
                print("%s - Can't delete, service [%s] unknown"%(service))
                continue

            if db[fn]['services'][service]['status']!=self.ST_DELETED:
                print("%s - rm: Can't remove file with non 'D' status (%s)!"\
                        %(fn,service))
                continue

            # Only change status if correctly deleted
            if self.sman.GetServiceObj(service).Remove(directory,fn):
                # Delete our service entry
                del db[fn]['services'][service]
                logger.debug('%s - deleted by service: %s'%(fn,service))
            else:
                logger.error('%s - Failed to delete by service: %s'%(fn,service))
                continue

        # Delete whole entry if no services manage it any more
        if len(db[fn]['services'].keys())==0:
            del db[fn]

        return True


    def _uploadFile(self,directory,fn,dentry,db,service):
        """Uploads file and changes status to 'S'. Looks
        up service name with service string
        """
        # Create a hash of the file
        if fn not in db:
            print("%s - Not in DB, must run 'add' first!"%(fn))
        else:
            # If already added, see if it's modified, only then
            # do another upload
            if db[fn]['services'][service]['status']==self.ST_UPTODATE:
                if not dentry['status']==self.ST_MODIFIED:
                    logger.info("%s - Up to date, skipping (%s)!"\
                            %(fn,service))
                    return False

            sobj=self.sman.GetServiceObj(service)
            # If nobody manages this file, just skip it
            if (not sobj):
                print("%s - Upload: No service of name [%s] found"%(fn,service))
                return
            # Only change status if correctly uploaded
            if sobj.Upload(directory,fn):
                # Write newest mtime/hash to indicate all is well
                db[fn]['mtime']=dentry['mtime']
                db[fn]['hash']=self._hashfile(os.path.join(directory,fn))
                db[fn]['services'][service]['status']=self.ST_UPTODATE
                logger.debug('%s - uploaded by service: %s'%(fn,sobj.GetName()))
                return True
            else:
                logger.error('%s - Failed to upload by service: %s'%(fn,sobj.GetName()))
                return False


        return False

    def _updateToDeleted(self,directory,fn,dentry,db,service):
        """Changes to status to 'D' as long as a handler exists,
        directory - DIR where stuff is happening
        fn - File name to be added
        dentry - dictionary entry as returned by GetStatus for this file
        db - pusher DB for this directory
        service - service to delete, None means all
        """
        # Create a hash of the file
        if fn not in db:
            print("%s - rm: not in DB, skipping!"%(fn))
            return

        services=self.sman.GetServices(fn)
        # If nobody manages this file, just skip it
        if (not services):
            print("%s - no manger of this file type found"%(fn))
            return

        if service:
            if db[fn]['services'].has_key(service):
                db[fn]['services'][service]['status']=self.ST_DELETED
            else:
                print("%s - Service %s doesn't exist, can't delete"%(fn,service))

            return

        # If we get here it means all services should delete
        for service in db[fn]['services']:
            db[fn]['services'][service]['status']=self.ST_DELETED

        return


    def _updateToAdded(self,directory,fn,dentry,db,service):
        """Changes to status to 'A' as long as a handler exists,
        also generates a hash
        directory - DIR where stuff is happening
        fn - File name to be added
        dentry - dictionary entry as returned by GetStatus for this file
        db - pusher DB for this directory
        service - None means all services, otherwise looks for service
        """
        services=self.sman.GetServices(fn)

        # If nobody manages this file, just skip it
        if services is None:
            print("%s - No services handle this file" %(fn))
            return

        # Build up list of names
        servicenames=[]
        for s in services:
            servicenames.append(s.GetName())

        if service is not None and service not in servicenames:
            print("%s - Requested service (%s) not available for this file"\
                    %(fn,service))
            return

        # If service is none, build list of all services
        # to perform this action on
        if service is None:
            servicelist=servicenames
        else:
            servicelist=[service]

        if not db.has_key(fn):
            # Since this is a new entry, populate with stuff
            # we got from GetSatus for this file (usually mtime)
            db[fn]=dentry
            del db[fn]['status'] # Delete this key we're not using
            db[fn]['services']={} # Empty dictionary of services
                                  # that manages this file + status

        # Now add the hash
        db[fn]['hash']=self._hashfile(os.path.join(directory,fn))

        # Now run through services and see if we should
        # perform actions
        for service in servicelist:
            if not db[fn]['services'].has_key(service):
                db[fn]['services'][service]={}
                db[fn]['services'][service]['status']=self.ST_ADDED
            else:
                print("%s - Already managed by service %s, maybe do a 'push'?"\
                        %(fn,service))

        logger.info('%s - managers: %s'%(fn,db[fn]['services'].keys()))

        return

    def _loadDB(self,directory):
        # See if we can open our DB file
        logger.debug("Trying to open pusher DB")
        try:
            db=json.load(open(os.path.join(directory,self.DB_FILE)))
        except IOError:
            logger.warning("Pusher DB not found")
            db={}

        # Hack to read old DB format where key 'services' didn't exist
        # We only supported flickr back then, so let's assume this
        for fn in db:
            if not db[fn].has_key('services'):
                db[fn]['services']={}
                db[fn]['services']['flickr']={}
                db[fn]['services']['flickr']['status']=db[fn]['status']

        return db

    def _saveDB(self,directory,db):
        # See if we can open our DB file
        logger.debug("Trying to save pusher DB")
        try:
            db=json.dump(db,open(os.path.join(directory,self.DB_FILE),'w'))
        except IOError:
            logger.warning("Pusher DB could not be saved")

    def PrintStatus(self,directory,files=None,service=None):
        dfiles,db=self.GetStatus(directory,files)
        keys=dfiles.keys()
        keys.sort()
        for fn in keys:
            if files and fn not in files:
                logging.debug('%s Skipping, for PrintStatus',fn)
                continue

            # Do we have this file in the DB?
            # if so, get the services
            if fn in db:
                has_service=db[fn].has_key('services')
            else:
                has_service=False

            # Don't print the pesky ./ dir 
            if directory!='.':
                fullfn=os.path.join(directory,fn)
            else:
                fullfn=fn

            if not has_service:
                print("%s %s"%(dfiles[fn]['status'],fullfn))
            else:
                service_str=''
                for service in db[fn]['services']:
                    service_str+=('%s[%s] '%(service,\
                            db[fn]['services'][service]['status']))

                print("%s %s (%s)"%(dfiles[fn]['status'],fullfn,\
                        service_str.strip()))
                
    def _hashfile(self,filename,blocksize=65536):
        """Hashes the file and returns hash"""
        logger.debug("Hashing file %s"%(filename))
        hasher=hashlib.sha256()
        afile=open(filename,'rb')
        buf=afile.read(blocksize)
        while len(buf) > 0:
            hasher.update(buf)
            buf = afile.read(blocksize)
        return hasher.hexdigest()


    def _findFiles(self,pushdir,files=None):
        """Find all files, return dict, where key is filename
        files[filename]=dictionary
        where dictionary has keys:
            ['mtime'] = modification time
        """
        
        #If no file list given, go through all
        if not files:
            files=os.listdir(pushdir)

        dfiles={}
        for fn in files:
            # Skip . files
            if fn[0]=='.':
                continue
            fullfile=os.path.join(pushdir,fn)
            d={}
            d['mtime']=os.path.getmtime(fullfile)
            dfiles[fn]=d

        return dfiles
        
        # Do hash if we have no DB
        #if not db:
            #logger.info("Hashing %d files"%(len(flickr_files)))
            #for d in flickr_files:
                #d['hash']=hashfile(os.path.join(pushdir,d['filename']))




import facebook
try:
    import facebook_login
except:
    import pypu.facebook_login as facebook_login

import sys
import os
import socket
import logging
import json
from pygeocoder import Geocoder
import exifread
import tempfile

try:
    import pusher_utils
except:
    import pypu.pusher_utils as pusher_utils

USER_ID = "me" # Will use user we are authenticated as

LOCATION_FILE='location.txt' # If no EXIF GPS data, use this to geotag
SET_FILE='sets.txt' # Sets photos belong to in directory
MEGAPIXEL_FILE='megapixels_fb.txt' # How many Mega Pixels to upload files as

logger=logging.getLogger('pusher')

class service_facebook:
    """Manages photo/video files on facebook. Any .jpg .mov .avi photos
    will be uploaded. Other special files also handled
    location.txt - Will use this location for photos unless photo is geotagged
    sets.txt - Add all files in this director to this set"""

    def __init__(self):
        self.name="fb"
        self.FB_CONFIG_FILES=[SET_FILE,MEGAPIXEL_FILE]
        self.FB_META_EXTENSIONS=['.title']
        # All MEDIA_EXTENSIONS will be uploaded to fb
        #self.FB_MEDIA_EXTENSIONS=['.jpg','.mov','.avi','.mp4']
        self.FB_MEDIA_EXTENSIONS=['.jpg'] # Videos not supported yet
        self.DB_FILE=".facebookdb"
        self.connected_to_fb=False

    def _connectToFB(self):
        """Establish the actual TCP connection to FB"""

        if self.connected_to_fb:
            logger.debug("Already connected to fb")
            return True

        logger.debug("Connecting to fb")

        token = facebook_login.get_fb_token()

        try:
            self.fb = facebook.GraphAPI(token)
        except:
            print("Couldn't connect to fb")
            return False

        self.connected_to_fb=True

        return True

    def GetName(self):
        return self.name

    def _isMediaFile(self, filename):
        ext=os.path.splitext(filename)[1].lower()
        if ext in self.FB_MEDIA_EXTENSIONS:
            return True
        return False

    def _isConfigFile(self,filename):
        """If this is a config file or meta file
        return true"""
        ext=os.path.splitext(filename)[1].lower()
        if filename in self.FB_CONFIG_FILES:
            return True
        elif ext in self.FB_META_EXTENSIONS:
            return True
        return False


    def KnowsFile(self,filename):
        """Looks at extension and decides if it knows
        how to manage this file"""
        if self._isMediaFile(filename) or self._isConfigFile(filename):
            return True
        return False

    def Upload(self,directory,filename):
        """Uploads/Updates/Replaces files"""
        if self._isMediaFile(filename):
            return self._upload_media(directory,filename)
        elif self._isConfigFile(filename):
            return self._update_config(directory,filename)

        print "Not handled!"
        return False

    def Remove(self,directory,filename):
        """Deletes files from fb"""
        if self._isMediaFile(filename):
            return self._remove_media(directory,filename)
        elif self._isConfigFile(filename):
            return True

        print "Not handled!"
        return False

    def _update_config(self,directory,filename):
        """Manages FB config files"""
        basefilename=os.path.splitext(filename)[0]
        ext=os.path.splitext(filename)[1].lower()
        #if filename==LOCATION_FILE:
            #return self._update_config_location(directory)
        #FIXME
        #elif filename==TAG_FILE:
            #return self._update_config_tags(directory)
        if filename==SET_FILE:
            print("%s - Moving photos to album"%(filename))
            return self._upload_media(directory,movealbum_request=True)
        elif filename==MEGAPIXEL_FILE:
            print("%s - Resizing photos"%(filename))
            return self._upload_media(directory,resize_request=True)
        elif ext in self.FB_META_EXTENSIONS:
            print("%s - Changing photo title"%(basefilename))
            return self._upload_media(directory,basefilename,changetitle_request=True)

        return False

    def _get_title(self,directory,filename):
        """Loads image title if any"""
        # =========== LOAD TITLE ========
        fullfile=os.path.join(directory,filename+'.title')
        try:
            logger.debug('trying to open [%s]'%(fullfile))
            _title=(open(fullfile).readline().strip())
            logger.debug("_updatemeta: %s - title is '%s'",filename,_title)
        except:
            _title=''

        return _title

    def _load_megapixels(self,directory):
        """Opens megapixel file, if contains '3.5' for instance,
        will scale all uploaded photos in directory this this size,
        the original photo is untouched. Returns None if
        file not found
        
        """
        #FIXME: should check if DB tracking file before using it

        fullfile=os.path.join(directory,MEGAPIXEL_FILE)
        try:
            mp=float(open(fullfile).readline())
            logger.debug("_load_megapixel: MP from file is %f",mp)
        except:
            logger.warning("Couldn't open image size file in %s, not scaling images"\
                    %(directory))
            return None

        return mp

    def _load_sets(self,directory):
        """Loads sets from set file and return
        as list of strings
        """
        # --- Read sets out of file
        _sets=[]
        try:
            fullfile=os.path.join(directory,SET_FILE)
            lsets=open(fullfile).readline().split(',')
            for tag in lsets:
                _sets.append(tag.strip())
        except:
            logger.error("No sets found in %s, FB needs an album name (%s)"\
                    %(directory,SET_FILE))
            sys.exit(1)

        return _sets

    def _createphotoset(self,myset):
        """Creates a photo set (album) on FB"""
        if not self._connectToFB():
            print("%s - Couldn't connect to fb"%(directory))
            return False
        
        logger.debug('fb: Creating photo set %s' %(myset))
        resp=self.fb.put_object(USER_ID,"albums",name=myset)

        if not resp.has_key('id'):
            logger.error("%s - fb: _createphotoset failed to create album",\
                    myset);
            return False
        else:
            return True

    def _get_album(self,directory):
        """
        Loads set name from SET_FILE, looks up album_id on fb, 
        it it doesn't exists, creates album. Returns album id and album name
        """
        if not self._connectToFB():
            print("%s - Couldn't connect to fb"%(directory))
            return None,None


        # Load sets from SET_FILE
        _sets=self._load_sets(directory)

        # Only grab the first set, FB supports only one set per photo
        myset=_sets[0]

        logger.debug("Getting album id for %s"%(myset))

        # Connect to fb and get dicionary of photosets
        psets=self._getphotosets()

        # create if it doesn't exist
        if myset not in psets:
            logger.info('set [%s] not in fb sets, will create set'%(myset))
            self._createphotoset(myset)
            # Now reaload photosets from fb
            psets=self._getphotosets()

        # Return the album id, album name
        return psets[myset]['id'],myset

    def _getphotosets_forphoto(self,pid):
        """Asks fb which photosets photo with
        given pid belongs to, returns list of
        photoset names"""

        resp=self.fb.photos_getAllContexts(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - fb: photos_getAllContext failed with status: %s",\
                    resp.attrib['stat']);
            return None

        lphotosets=[]
        for element in resp.findall('set'):
            lphotosets.append(element.attrib['title'])

        logger.debug('%s - belongs to these photosets %s',pid,lphotosets)

        return lphotosets

    def _getphoto_originalsize(self,pid):
        """Asks fb for photo original size
        returns tuple with width,height
        """
        logger.debug('%s - Getting original size from fb'%(pid))
        i=self.fb.get_object(pid)
        width=i['images'][0]['width']
        height=i['images'][0]['height']
        return (width,height)


    def _getphoto_location(self,pid):
        """Asks fb for photo location information
        returns tuple with lat,lon,accuracy
        """

        logger.debug('%s - Getting location from fb'%(pid))

        lat=None
        lon=None
        accuracy=None

        resp=self.fb.photos_geo_getLocation(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - fb: photos_geo_getLocation failed with status: %s",\
                    resp.attrib['stat']);
            return (None,None,None)
        
        for location in resp.find('photo'):
            lat=location.attrib['latitude']
            lon=location.attrib['longitude']
            accuracy=location.attrib['accuracy']
            
                
        return (lat,lon,accuracy)

    def _remove_media(self,directory,files=None):
        """Removes specified files from fb"""
        # Connect if we aren't already
        if not self._connectToFB():
            logger.error("%s - Couldn't connect to fb")
            return False

        db=self._loadDB(directory)
        # If no files given, use files from DB in dir
        if not files:
            files=db.keys()

        #If only one file given, make it a list
        if isinstance(files,basestring):
            files=[files]

        for fn in files:
            print("%s - Deleting from fb [local copy intact]"%(fn))

            try:
                pid=db[fn]['photoid']
            except:
                logger.debug("%s - Was never in fb DB"%(fn))
                continue
            try:
                self.fb.delete_object(pid)
            except facebook.GraphAPIError as e:
                print("%s - fb: delete failed with status: %s:%s"\
                        %(fn,e.type,e.message))
                return False

            logger.debug('Removing %s from fb DB'%(fn))
            del db[fn]
            self._saveDB(directory,db)

        return True

    def _upload_media(self,directory,files=None,resize_request=None, \
            movealbum_request=None,changetitle_request=None):
        """Uploads media file to FB, returns True if
        uploaded successfully, Will replace 
        if already uploaded, If megapixels > 0, will
        scale photos before upload
        If no filename given, will go through all files in DB"""
        # Connect if we aren't already
        if not self._connectToFB():
            logger.error("%s - Couldn't connect to fb")
            return False

        _megapixels=self._load_megapixels(directory)

        # Get an album ID (create album if not exists)
        _album_id,_album_name=self._get_album(directory)

        if not _megapixels:
            mpstring="original"
        else:
            mpstring=("%0.1f MP"%(_megapixels))

        # If no files given, use files from DB in dir
        if not files:
            db=self._loadDB(directory)
            files=db.keys()

        #If only one file given, make it a list
        if isinstance(files,basestring):
            files=[files]

        files.sort()

        for filename in files:

            # Get title here if any
            title=self._get_title(directory,filename)

            if title:
                print("%s - Uploading to fb, album[%s] size=%s title=%s"\
                        %(filename,_album_name,mpstring,title))
            else:
                print("%s - Uploading to fb, album[%s] size=%s"\
                        %(filename,_album_name,mpstring))

            status=self._upload_or_replace_fb(directory,filename, \
                    _album_id, _megapixels,resize_request,movealbum_request,\
                    changetitle_request,title)
            if not status:
                return False

        return True

    def _upload_or_replace_fb(self,directory,fn,_album_id,\
            _megapixels=None,resize_request=None,movealbum_request=None,\
            changetitle_request=None,_title=None):
        """Does the actual upload to fb.
        if resize_request, will resize picture only if
        it already exists and the geometry on fb doesn't match
        what we want,
        returns (status)"""
        # We should check here if 
        db=self._loadDB(directory)

        # If resize request, make tempfile and
        # resize.
        if _megapixels:
            fp = tempfile.NamedTemporaryFile()
            fullfile_resized=fp.name
            logger.debug("tempfile for resized is %s"%(fp.name))

        fullfile=os.path.join(directory,fn)

        # If JPEG, then resize
        ext=os.path.splitext(fullfile)[1].lower()
        if ext=='.jpg':
            isJPG=True
        else:
            isJPG=False

        # If already in DB, remove first, then overwrite
        if fn in db:
            pid=db[fn]['photoid']
            if resize_request and isJPG:
                logger.info("fb: Resize request for %s",fn)
                if self._already_resized_on_fb(fullfile,pid,_megapixels):
                    logger.debug("%s - Already in DB and resized, skipping",fn)
                    return True
            elif movealbum_request:
                logger.info("fb: Move album request for %s",fn)
                if self._already_in_album(fullfile,pid,_album_id):
                    logger.debug("%s - Already in DB and in correct album, skipping",fn)
                    return True
            elif changetitle_request:
                logger.info("fb: Change title request for %s",fn)
                if self._title_uptodate(fullfile,pid,_title):
                    logger.debug("%s - Already in DB and title up to date, skipping",fn)
                    return True


            # --- If we are here it means photo should be updated.
            # With FB graph API this means removing the photo
            # and uploading with new meta data.
            logger.debug("%s - Already in DB, removing first",fn)
            if not self._remove_media(directory,fn):
                logger.error("%s - fb: couldn't replace (remove) file\n",fn)
                return False

        # Do we have to resize?
        if _megapixels and isJPG:
            if pusher_utils.resize_image(fullfile,fullfile_resized,_megapixels):
                logger.debug("%s resized to %s successfully"\
                        %(fullfile,fullfile_resized))
                fullfile=fullfile_resized
            else:
                logger.warning("%s couldn't resize, uploading original"\
                        %(fullfile))

        logger.debug("Upload %s to fb, album=%s, title='%s'",\
                fn,_album_id,_title)

        # We can get a place id by doing a search
        # http://graph.facebook.com/search?type=city&center=37,-122&distance=1000

        # Do the actual upload
        resp=self.fb.put_photo(open(fullfile),\
                message=_title,album_id=_album_id,\
                )
                #place='106377336067638'\

        logger.debug("%s - Upload response is : %s"%(fn,resp))

        if not resp.has_key('id'):
            print("%s - fb: upload failed", fn)
            return False

        pid=resp['id']
        db[fn]={}
        db[fn]['photoid']=pid
        logger.debug("%s - fb: uploaded with photoid %s",fn,pid);

        self._saveDB(directory,db)
        return True

    def _title_uptodate(self,fullfile,pid,_title):
        """Check fb photo title against provided title,
        returns true if they match"""
        i=self.fb.get_object(pid)
        if i.has_key('name'):
            if _title == i['name']:
                return True

        return False

    def _already_in_album(self,fullfile,pid,album_id):
        """Check to see if photo with given pid is already
        in the album_id, returns true if this is the case
        """

        logger.debug("fb: Checking if pid %s in album %s",pid,album_id)
        pid_in_album=[]
        # Get all photos in album
        photos = self.fb.get_connections(str(album_id),"photos")['data']

        # Get all pids in fb album
        for photo in photos:
            pid_in_album.append(photo['id'])

        logger.debug("fb: album %d contains these photos: %s",album_id,pid_in_album)

        # Check if our pid matches
        if pid in pid_in_album:
            return True

        return False

    def _already_resized_on_fb(self,fn,pid,_megapixels):
        """Checks if image file (fn) with photo_id (pid) has already
        been resized on fb. If so, returns True"""
        logger.debug("%s - resize requested"%(fn))
        # Get width/height from fb
        width_fb,height_fb=self._getphoto_originalsize(pid)
        # Now compute what image will be if we resize it
        new_width,new_height=pusher_utils.resize_compute_width_height(\
                fn,_megapixels)
        logger.debug("%s - fb %d/%d, current %d/%d"\
                %(fn,width_fb,height_fb,new_width,new_height))
        # Check both cases since FB sometimes rotates photos
        if width_fb==new_width and height_fb==new_height:
            return True
        elif width_fb==new_height and height_fb==new_width:
            return True

        return False

    def _getphotosets(self):
        """Returns dictionary of photosets retrieved from fb
        d['title']['number_photos'] : Number of photos
        d['title']['id']            : ID of photoset
        d['title']['photo_id']      : ID of primary photo
        d['title']['url']           : URL to photoset
        """
        sets={}
        if not self._connectToFB():
            print("Couldn't connect to fb")
            return sets

        # Gets facebook albums
        psets = self.fb.get_connections(USER_ID,"albums")['data']
        for myset in psets:
            key=myset['name']
            sets[key]={}
            try:
                sets[key]['number_photos']=int(myset['count'])
            except:
                sets[key]['number_photos']=0
            try:
                sets[key]['photo_id']=myset['cover_photo']
            except:
                sets[key]['photo_id']=None

            sets[key]['id']=int(myset['id'])
            sets[key]['url']=myset['link']

        return sets

    def PrintSets(self):
        """Prints set name and number of photos in set"""
        sets=self._getphotosets()
        for setname in sets:
            print("%s [%d]"%(setname,sets[setname]['number_photos']))
    
    def _loadDB(self,directory):
        # See if we can open our DB file
        logger.debug("Trying to open fb DB")
        try:
            db=json.load(open(os.path.join(directory,self.DB_FILE)))
        except IOError:
            db={}

        return db

    def _saveDB(self,directory,db):
        # See if we can open our DB file
        logger.debug("Trying to save fb DB")
        try:
            db=json.dump(db,open(os.path.join(directory,self.DB_FILE),'w'))
        except IOError:
            logger.warning("Pusher DB could not be saved")



import sys
import os
import socket
import flickrapi
import logging
import json
from pygeocoder import Geocoder
import tempfile

try:
    import pusher_utils
except:
    import pypu.pusher_utils as pusher_utils


api_key = '8792333a5c26b561ccff7981cfb80e81'
api_secret='b186204612137f1f'

myid='105164994@N02' # My flickr ID

UPLOAD_RETRY_MAX=3
SOCKET_TIMEOUT_SECONDS=120 # Timeout for urllib2 http post
LOCATION_FILE='location.txt' # If no EXIF GPS data, use this to geootag
SET_FILE='sets.txt' # Sets photos belong to in directory
TAG_FILE='tags.txt' # List of tags for photos in directory
MEGAPIXEL_FILE='megapixels.txt' # How many Mega Pixels to upload files as

logger=logging.getLogger('pusher')


class service_flickr:
    """Manages photo/video files on flickr. Any .jpg .mov .avi photos
    will be uploaded. Other special files also handled
    location.txt - Will use this location for photos unless photo is geotagged
    tags.txt - Tags for all photos in this directory
    sets.txt - Add all files in this director to this set"""

    def __init__(self):
        # Sometimes flickrapi times out, here
        # we set the gobal timeout in seconds
        #socket.setdefaulttimeout(SOCKET_TIMEOUT_SECONDS)
        self.name="flickr"
        self.FLICKR_CONFIG_FILES=[LOCATION_FILE,SET_FILE,TAG_FILE,MEGAPIXEL_FILE]
        self.FLICKR_META_EXTENSIONS=['.title']
        # All MEDIA_EXTENSIONS will be uploaded to flickr
        self.FLICKR_MEDIA_EXTENSIONS=['.jpg','.mov','.avi','.mp4']
        self.DB_FILE=".flickrdb"
        self.connected_to_flickr=False

    def _connectToFlickr(self):
        """Establish the actual TCP connection to Flickr"""

        if self.connected_to_flickr:
            logger.debug("Already connected to flickr")
            return True

        logger.debug("Connecting to flickr")
        # Do flickr authentication
        _flickr = flickrapi.FlickrAPI(api_key, api_secret,format='etree')
        try:
            (token, frob) = _flickr.get_token_part_one(perms='delete')
        except:
            print("Couldn't connect to flickr")
            return False

        if not token: raw_input("Press ENTER after you authorized this program")
        _flickr.get_token_part_two((token, frob))

        self.flickr=_flickr;
        
        self.connected_to_flickr=True

        return True

    def GetName(self):
        return self.name

    def _isMediaFile(self, filename):
        ext=os.path.splitext(filename)[1].lower()
        if ext in self.FLICKR_MEDIA_EXTENSIONS:
            return True
        return False

    def _isConfigFile(self,filename):
        """If this is a config file or meta file
        return true"""
        ext=os.path.splitext(filename)[1].lower()
        if filename in self.FLICKR_CONFIG_FILES:
            return True
        elif ext in self.FLICKR_META_EXTENSIONS:
            return True
        return False


    def KnowsFile(self,filename):
        """Looks at extension and decides if it knows
        how to manage this file"""
        if self._isMediaFile(filename) or self._isConfigFile(filename):
            return True
        return False

    def Upload(self,directory,filename):
        """Uploads/Updates/Replaces files"""
        if self._isMediaFile(filename):
            return self._upload_media(directory,filename)
        elif self._isConfigFile(filename):
            return self._update_config(directory,filename)

        print "Not handled!"
        return False

    def Remove(self,directory,filename):
        """Deletes files from flickr"""
        if self._isMediaFile(filename):
            return self._remove_media(directory,filename)
        elif self._isConfigFile(filename):
            return True

        print "Not handled!"
        return False

    def _update_config(self,directory,filename):
        """Manages FLICKR config files"""
        basefilename=os.path.splitext(filename)[0]
        ext=os.path.splitext(filename)[1].lower()
        if filename==LOCATION_FILE:
            print("%s - Updating geotag information"%(LOCATION_FILE))
            return self._update_config_location(directory)
        elif filename==TAG_FILE:
            print("%s - Updating tags"%(TAG_FILE))
            return self._update_config_tags(directory)
        elif filename==SET_FILE:
            print("%s - Updating sets"%(SET_FILE))
            return self._update_config_sets(directory)
        elif filename==MEGAPIXEL_FILE:
            print("%s - Updating photo size"%(MEGAPIXEL_FILE))
            return self._upload_media(directory,resize_request=True)
        elif ext in self.FLICKR_META_EXTENSIONS:
            return self._update_meta(directory,basefilename)

        return False

    def _update_meta(self,directory,filename):
        """Opens up filename.title and
        filename.description, updates on flickr"""

        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(directory))
            return False

        db = self._loadDB(directory)

        # Look up photo id for this photo
        pid=db[filename]['photoid']

        # =========== LOAD TITLE ========
        fullfile=os.path.join(directory,filename+'.title')
        try:
            logger.debug('trying to open [%s]'%(fullfile))
            _title=(open(fullfile).readline().strip())
            logger.debug("_updatemeta: %s - title is %s",filename,_title)
        except:
            _title=''

        # =========== LOAD DESCRIPTION ========
        fullfile=os.path.join(directory,filename+'.description')
        try:
            _description=(open(fullfile).readline().strip())
            logger.debug("_updatemeta: %s - description is %s",filename,_description)
        except:
            _description=''

        logger.info('%s - updating metadata (title=%s) (description=%s)'\
                %(filename,_title,_description))
        resp=self.flickr.photos_setMeta(photo_id=pid,title=_title,\
                description=_description)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_setTags failed with status: %s",\
                    resp.attrib['stat']);
            return False
        else:
            return True



    def _load_tags(self,directory):
        """Loads tags from tag file and return
        as flickr api compatible string
        """
        #FIXME: should check if DB tracking file before using it
        # --- Read tags out of file
        _tags=''
        try:
            fullfile=os.path.join(directory,TAG_FILE)
            ltags=open(fullfile).readline().split(',')
            _tags=''' '''

            for tag in ltags:
                _tags+='''"'''+tag.strip()+'''" '''
            _tags=_tags.strip()
        except:
            logger.info("No tags found in %s"%(directory))

        return _tags

    def _load_megapixels(self,directory):
        """Opens megapixel file, if contains '3.5' for instance,
        will scale all uploaded photos in directory this this size,
        the original photo is untouched. Returns None if
        file not found
        
        """
        #FIXME: should check if DB tracking file before using it

        fullfile=os.path.join(directory,MEGAPIXEL_FILE)
        try:
            mp=float(open(fullfile).readline())
            logger.debug("_load_megapixel: MP from file is %f",mp)
        except:
            logger.warning("Couldn't open image size file in %s, not scaling images"\
                    %(directory))
            return None

        return mp

    def _load_sets(self,directory):
        """Loads sets from set file and return
        as list of strings
        """
        # --- Read sets out of file
        _sets=[]
        try:
            fullfile=os.path.join(directory,SET_FILE)
            lsets=open(fullfile).readline().split(',')
            for tag in lsets:
                _sets.append(tag.strip())
        except:
            logger.info("No sets found in %s"%(directory))

        return _sets

    def _createphotoset(self,myset,primary_photoid):
        """Creates a photo set on Flickr"""
        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(directory))
            return False
        
        logger.debug('Creating photo set %s with prim photo %s'\
                %(myset,primary_photoid))
        resp=self.flickr.photosets_create(title=myset,\
                primary_photo_id=primary_photoid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_setTags failed with status: %s",\
                    resp.attrib['stat']);
            return False
        else:
            return True

    def _update_config_sets(self,directory,files=None):
        """
        Loads set information from file and updates on flickr, 
        only reads first line. Format is comma separated eg.
        travel, 2010, South Africa, Pretoria
        If files is None, will update all files in DB, otherwise
        will only update files that are in the flickr DB and files list
        """
        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(directory))
            return False

        # Load sets from SET_FILE
        _sets=self._load_sets(directory)

        # Connect to flickr and get dicionary of photosets
        psets=self._getphotosets()

        db = self._loadDB(directory)

        # To create a set, one needs to pass it the primary
        # photo to use, let's open the DB and load the first
        # photo
        primary_pid=db[db.keys()[0]]['photoid']

        # Loop through all sets, create if it doesn't exist
        for myset in _sets:
            if myset not in psets:
                logger.info('set [%s] not in flickr sets, will create set'%(myset))
                self._createphotoset(myset,primary_pid)

        # Now reaload photosets from flickr
        psets=self._getphotosets()

        # --- Load DB of photos, and update them all with new tags
        for fn in db:
            # --- If file list provided, skip files not in the list
            if files and fn not in files:
                continue

            pid=db[fn]['photoid']

            # Get all the photosets this photo belongs to
            psets_for_photo=self._getphotosets_forphoto(pid)

            for myset in _sets:
                if myset in psets_for_photo:
                    logger.debug("%s - Already in photoset [%s] - skipping"%(fn,myset))
                    continue
                logger.info("%s [flickr] Adding to set [%s]" %(fn,myset))
                psid=psets[myset]['id']
                logger.debug("%s - Adding to photoset %s"%(fn,psid))
                resp=self.flickr.photosets_addPhoto(photoset_id=psid,photo_id=pid)
                if resp.attrib['stat']!='ok':
                    logger.error("%s - flickr: photos_addPhoto failed with status: %s",\
                            resp.attrib['stat']);
                    return False

            # Go through all sets flickr says this photo belongs to and
            # remove from those sets if they don't appear in SET_FILE
            for pset in psets_for_photo:
                if pset not in _sets:
                    psid=psets[pset]['id']
                    logger.info("%s [flickr] Removing from set [%s]" %(fn,pset))
                    logger.debug("%s - Removing from photoset %s"%(fn,psid))
                    resp=self.flickr.photosets_removePhoto(photoset_id=psid,photo_id=pid)
                    if resp.attrib['stat']!='ok':
                        logger.error("%s - flickr: photossets_removePhoto failed with status: %s",\
                                resp.attrib['stat']);
                        return False

        return True


    def _getphotosets_forphoto(self,pid):
        """Asks flickr which photosets photo with
        given pid belongs to, returns list of
        photoset names"""

        resp=self.flickr.photos_getAllContexts(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_getAllContext failed with status: %s",\
                    resp.attrib['stat']);
            return None

        lphotosets=[]
        for element in resp.findall('set'):
            lphotosets.append(element.attrib['title'])

        logger.debug('%s - belongs to these photosets %s',pid,lphotosets)

        return lphotosets

    def _getphoto_originalsize(self,pid):
        """Asks flickr for photo original size
        returns tuple with width,height
        """

        logger.debug('%s - Getting original size from flickr'%(pid))

        width=None
        height=None

        resp=self.flickr.photos_getSizes(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_getSizes failed with status: %s",\
                    resp.attrib['stat']);
            return (None,None)
        
        for size in resp.find('sizes').findall('size'):
            if size.attrib['label']=="Original":
                width=int(size.attrib['width'])
                height=int(size.attrib['height'])
                logger.debug('Found pid %s original size of %s,%s'\
                    %(pid,width,height))
                
        return (width,height)

    def _getphoto_location(self,pid):
        """Asks flickr for photo location information
        returns tuple with lat,lon,accuracy
        """

        logger.debug('%s - Getting location from flickr'%(pid))

        lat=None
        lon=None
        accuracy=None

        resp=self.flickr.photos_geo_getLocation(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_geo_getLocation failed with status: %s",\
                    resp.attrib['stat']);
            return (None,None,None)
        
        for location in resp.find('photo'):
            lat=location.attrib['latitude']
            lon=location.attrib['longitude']
            accuracy=location.attrib['accuracy']
            
                
        return (lat,lon,accuracy)

    def _getphoto_url(self,pid,size='n'):
        """Returns URL to photo, optionally can link
        to different size images
        Size suffixes according to:
        http://www.flickr.com/services/api/misc.urls.html

        s   small square 75x75
        q   large square 150x150
        t   thumbnail, 100 on longest side
        m   small, 240 on longest side
        n   small, 320 on longest side
        -   medium, 500 on longest side
        z   medium 640, 640 on longest side
        c   medium 800, 800 on longest side
        b   large, 1024 on longest side*
        o   original image, either a jpg, gif or png, depending on source format

        """
        p=self._getphoto_information(pid)
        farm=p['farm']
        server=p['server']
        secret=p['secret']
        url="http://farm%s.staticflickr.com/%s/%s_%s_%s.jpg"\
            %(farm,server,pid,secret,size)

        return url
        
    def _getphoto_information(self,pid):
        """Asks flickr for photo information
        returns dictionary with attributes

        {'dateuploaded': '1383410793',
         'farm': '3',
         'id': '10628709834',
         'isfavorite': '0',
         'license': '0',
         'media': 'photo',
         'originalformat': 'jpg',
         'originalsecret': 'b60f4f675f',
         'rotation': '0',
         'safety_level': '0',
         'secret': 'a4c96e996b',
         'server': '2823',
         'views': '1',
         'title': 'Image title'
         }

        """
        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(directory))
            return False

        d={}

        logger.debug('%s - Getting photo information from flickr'%(pid))

        resp=self.flickr.photos_getInfo(photo_id=pid)
        if resp.attrib['stat']!='ok':
            logger.error("%s - flickr: photos_getInfo failed with status: %s",\
                    resp.attrib['stat']);
            return None
        
        p=resp.find('photo')
        p.attrib['title']=p.find('title').text

        return p.attrib

    def _update_config_tags(self,directory,files=None):
        """
        Loads tags information from file and updates on flickr, 
        only reads first line. Format is comma separated eg.
        travel, 2010, South Africa, Pretoria
        If files is None, will update all files in DB, otherwise
        will only update files that are in the flickr DB and files list
        """
        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(directory))
            return False

        logger.debug("Updating tags in %s"%(directory))

        _tags=self._load_tags(directory)

        # --- Load DB of photos, and update them all with new tags
        db = self._loadDB(directory)
        for fn in db:
            # --- If file list provided, skip files not in the list
            if files and fn not in files:
                logger.debug('%s [flickr] Skipping, tag update',fn)
                continue

            logger.info("%s [flickr] Updating tags [%s]" %(fn,_tags))
            pid=db[fn]['photoid']
            resp=self.flickr.photos_setTags(photo_id=pid,tags=_tags)
            if resp.attrib['stat']!='ok':
                logger.error("%s - flickr: photos_setTags failed with status: %s",\
                        resp.attrib['stat']);
                return False
            else:
                return True
        return False

    def _update_config_location(self,directory,files=None):
        """Loads location and applies to all files in given
        files list (or if none, all files in flickr DB)

        google reverse GEO to figure out location lat/long
        file can contain things like:
        Australia
        Sydney, Australia
        Holcomb Campground, California.

        If image already has Lat/Lon in EXIF, it's location will
        not be updated.

        FIXME: Could save location lookup results in .file
        such that other files don't need to do it again
        """
        if not self._connectToFlickr():
            print("%s - Couldn't connect to flickr"%(filename))
            return False


        # --- Read location out of file
        fullfile=os.path.join(directory,LOCATION_FILE)
        try:
            location=open(fullfile).readline().strip()
        except:
            logger.info('No location information found');
            return False

        logger.debug('Setting location information : %s'%(location))

        # ---- Now do reverse geocoding
        try:
            results = Geocoder.geocode(location)
        except:
            logger.error("Couldn't find lat/lon for %s"%(location))
            return False

        #logger.debug(results.raw)
        logger.debug('google says location is: %s'%(results[0]))
        _lat,_lon=results[0].coordinates
        placename=results[0]

        # --- Load DB of photos, and update them all with new location
        db = self._loadDB(directory)
        for fn in db:
            # --- If file list provided, skip files not in the list
            if files and fn not in files:
                continue

            logger.debug('Checking %s for location change'%(fn))
            exif_lat,exif_lon=pusher_utils.getexif_location(directory,fn)
            if exif_lat and exif_lon:
                logger.info("%s [flickr] EXIF GPS found (%f,%f) - skipping"\
                        %(fn,exif_lat,exif_lon))
                continue
            else:
                logger.info("%s - GPS: no position, using location file"%(fn))

            logger.info("%s [flickr] Updating loc to %f,%f [%s]"\
                    %(fn,_lat,_lon,placename))
            pid=db[fn]['photoid']
            resp=self.flickr.photos_geo_setLocation(photo_id=pid,lat=_lat,lon=_lon)
            if resp.attrib['stat']!='ok':
                logger.error("%s - flickr: geo_setLocation failed with status: %s",\
                        resp.attrib['stat']);
                return False

        return True

    def _remove_media(self,directory,files=None):
        """Removes specified files from flickr"""
        # Connect if we aren't already
        if not self._connectToFlickr():
            logger.error("%s - Couldn't connect to flickr")
            return False

        db=self._loadDB(directory)
        # If no files given, use files from DB in dir
        if not files:
            files=db.keys()

        #If only one file given, make it a list
        if isinstance(files,basestring):
            files=[files]

        for fn in files:
            print("%s - Deleting from flickr [local copy intact]"%(fn))

            try:
                pid=db[fn]['photoid']
            except:
                logger.debug("%s - Was never in flickr DB"%(fn))
                continue
            resp=self.flickr.photos_delete(photo_id=pid,format='etree')
            if resp.attrib['stat']!='ok':
                print("%s - flickr: delete failed with status: %s",\
                        resp.attrib['stat']);
                return False
            else:
                logger.debug('Removing %s from flickr DB'%(fn))
                del db[fn]
                self._saveDB(directory,db)

        return True

    def _upload_media(self,directory,files=None,resize_request=None):
        """Uploads media file to FLICKR, returns True if
        uploaded successfully, Will replace 
        if already uploaded, If megapixels > 0, will
        scale photos before upload
        If no filename given, will go through all files in DB"""
        # Connect if we aren't already
        if not self._connectToFlickr():
            logger.error("%s - Couldn't connect to flickr")
            return False

        _tags=self._load_tags(directory)
        _megapixels=self._load_megapixels(directory)


        # If no files given, use files from DB in dir
        if not files:
            db=self._loadDB(directory)
            files=db.keys()

        #If only one file given, make it a list
        if isinstance(files,basestring):
            files=[files]

        files.sort()

        for filename in files:
            #FIXME: If this fails, should send a list
            # to Upload() about which files DID make it,
            # so we don't have to upload it again!

            status,replaced=self._upload_or_replace_flickr(directory,filename, \
                    _tags, _megapixels,resize_request)
            if not status:
                return False
            # If uploaded OK, update photo properties, tags
            # already taken care of - only update if
            # this is a new photo (eg, if it was replaced
            # then we don't need to do this
            if not replaced:
                self._update_config_location(directory,filename)
                self._update_config_sets(directory,filename)

        return True

    def _upload_or_replace_flickr(self,directory,fn,_tags=None,\
            _megapixels=None,resize_request=None):
        """Does the actual upload to flickr.
        if resize_request, will resize picture only if
        it already exists and the geometry on flickr doesn't match
        what we want,
        returns (status,replaced)"""
        # We should check here if 
        db=self._loadDB(directory)

        status=False
        replaced=False

        if not _megapixels:
            mpstring="original"
        else:
            mpstring=("%0.1f MP"%(_megapixels))

        # If resize request, make tempfile and
        # resize.
        if _megapixels:
            fp = tempfile.NamedTemporaryFile()
            fullfile_resized=fp.name
            logger.debug("tempfile for resized is %s"%(fp.name))

        fullfile=os.path.join(directory,fn)

        ext=os.path.splitext(fullfile)[1].lower()
        # If JPEG, then resize
        if ext=='.jpg':
            isJPG=True
        else:
            isJPG=False
            

        # Possibly scale before uploading
        if fn not in db:
            # Do we have to resize?
            if _megapixels and isJPG:
                if pusher_utils.resize_image(fullfile,fullfile_resized,_megapixels):
                    logger.debug("%s resized to %s successfully"\
                            %(fullfile,fullfile_resized))
                    fullfile=fullfile_resized
                else:
                    logger.warning("%s couldn't resize, uploading original"\
                            %(fullfile))

            logger.debug("Upload %s to flickr, tags=%s",fn,_tags)

            print("%s - Uploading to flickr, tags[%s] size=%s"\
                    %(fn,_tags,mpstring))

            # Do the actual upload
            uplxml=self.flickr.upload(filename=fullfile,\
                    #title=_title,\
                    tags=_tags,\
                    format='etree')

            if uplxml.attrib['stat']!='ok':
                print("%s - flickr: upload failed with status: %s",\
                        fn,uplxml.attrib['stat']);
                status=False
                replaced=False
                return status,replaced

            pid=uplxml.find('photoid').text
            db[fn]={}
            db[fn]['photoid']=pid
            logger.debug("%s - flickr: uploaded with photoid %s",fn,pid);
            status=True
            replaced=False
        else:
            # File already exists, let's replace it.
            pid=db[fn]['photoid']
            # If this is an actual resize request,
            # go check geometry on flickr and resize
            # if it hasn't been already
            if resize_request and isJPG:
                if self._already_resized_on_flickr(fullfile,pid,_megapixels):
                    status=True
                    replaced=True
                    logger.info(\
                            '%s - flickr: already correct size - skipping'\
                            %(fn))
                    # File already resized, skip it.
                    return status,replaced

            # If megapixels given, will resize image
            if _megapixels and isJPG:
                if pusher_utils.resize_image(fullfile,fullfile_resized,_megapixels):
                    logger.debug("%s resized to %s successfully"\
                            %(fullfile,fullfile_resized))
                    fullfile=fullfile_resized
                else:
                    logger.warning("%s couldn't resize, uploading original"\
                            %(fullfile))
                    
            logger.info("%s - Replace on flickr pid=%s",fn,pid)
            uplxml=self.flickr.replace(filename=fullfile,photo_id=pid)
            if uplxml.attrib['stat']!='ok':
                print("%s - flickr: replace failed with status: %s",\
                        uplxml.attrib['stat']);
                status=False
                replaced=False
                return status,replaced
            else:
                replaced=True
                status=True

        # We should check here if 
        self._saveDB(directory,db)
        return status,replaced

    def _already_resized_on_flickr(self,fn,pid,_megapixels):
        """Checks if image file (fn) with photo_id (pid) has already
        been resized on flickr. If so, returns True"""
        logger.debug("%s - resize requested"%(fn))
        # Get width/height from flickr
        width_flickr,height_flickr=self._getphoto_originalsize(pid)
        # Now compute what image will be if we resize it
        new_width,new_height=pusher_utils.resize_compute_width_height(\
                fn,_megapixels)
        if width_flickr==new_width and height_flickr==new_height:
            return True
        # Also return true if image couldn't be resized
        elif not new_width:
            return True
        return False

    def _getphotosets(self):
        """Returns dictionary of photosets retrieved from flickr
        d['title']['number_photos'] : Number of photos
        d['title']['id']            : ID of photoset
        d['title']['photo_id']      : ID of primary photo
        d['title']['url']           : URL to photoset
        """
        sets={}
        if not self._connectToFlickr():
            print("Couldn't connect to flickr")
            return sets

        psets = self.flickr.photosets_getList(user_id=myid)
        for myset in psets.find('photosets').findall('photoset'):
            key=myset.find('title').text
            sets[key]={}
            sets[key]['number_photos']=int(myset.attrib['photos'])
            sets[key]['photo_id']=(myset.attrib['primary'])
            sets[key]['id']=int(myset.attrib['id'])
            sets[key]['url']='http://www.flickr.com/photos/%s/sets/%d/'\
                    %(myid,sets[key]['id'])

        return sets

    def PrintSets(self):
        """Prints set name and number of photos in set"""
        sets=self._getphotosets()
        for setname in sets:
            print("%s [%d]"%(setname,sets[setname]['number_photos']))
    
    def _loadDB(self,directory):
        # See if we can open our DB file
        logger.debug("Trying to open flickr DB")
        try:
            db=json.load(open(os.path.join(directory,self.DB_FILE)))
        except IOError:
            db={}

        return db

    def _saveDB(self,directory,db):
        # See if we can open our DB file
        logger.debug("Trying to save flickr DB")
        try:
            db=json.dump(db,open(os.path.join(directory,self.DB_FILE),'w'))
        except IOError:
            logger.warning("Pusher DB could not be saved")



import os
import exifread
from PIL import Image # requires pillow package in gentoo
from math import sqrt
import logging

logger=logging.getLogger('pusher')

def resize_image(fullfile,fullfile_resized,_megapixels):
    """Resizes image (fullfile), saves to fullfile_resized. Image
    aspect ratio is conserved, will be scaled to be close to _megapixels in 
    size. Eg if _megapixels=2, will resize 2560x1920 so each dimension
    is scaled by ((2**(20+1*MP))/float(2560*1920))**2"""
    
    logger.debug("%s - Resizing to %s MP"%(fullfile,_megapixels))

    img = Image.open(fullfile)
    width,height=img.size
    current_megapixels=width*height/(2.0**20)

    # Compute new width and height for image
    new_width,new_height=resize_compute_width_height(\
            fullfile,_megapixels)

    # Not scaling
    if not new_width:
        logger.debug("%s - NOT Resizing, scale is > 1"%(fullfile))
        return False

    logger.info("%s - Resizing image from %0.1f to %0.1f MP (%dx%d) to (%dx%d)"\
            %(fullfile,current_megapixels,_megapixels,width,height,new_width,new_height))
    # Resize the image
    imageresize = img.resize((new_width,new_height), Image.ANTIALIAS)
    #imageresize.save(fullfile_resized, 'JPEG', quality=75)
    #FIXME: What quality to save as?
    imageresize.save(fullfile_resized, 'JPEG')

    # ---- Transfer over EXIF info ----
    try:
        update_exif_GEXIV2(fullfile,fullfile_resized)
    except:
        logger.debug('GEXIV2 exif update failed, trying pyexiv2')
        update_exif_pyexiv2(fullfile,fullfile_resized)

    return True

def update_exif_pyexiv2(oldfile, newfile):
    import pyexiv2

    # Figure out dimensions
    imgresize = Image.open(newfile)
    
    # copy EXIF data
    oldmeta = pyexiv2.ImageMetadata(oldfile)
    oldmeta.read()

    newmeta = pyexiv2.ImageMetadata(newfile)
    newmeta.read()
    # read metadata of the new file

    oldmeta.copy(newmeta)

    # set EXIF image size info to resized size
    newmeta["Exif.Photo.PixelXDimension"] = (imgresize.size[0])
    newmeta["Exif.Photo.PixelYDimension"] = (imgresize.size[1])

    newmeta.write()

def update_exif_GEXIV2(oldfile,newfile):
    """Transfers oldfile's exif to newfile's exif and
    updates the width/height EXIF fields"""

    # Requires gexiv2 and pygobject package in gentoo 
    # (USE=introspection)
    from gi.repository import GExiv2 

    # exif of orginal image
    exif = GExiv2.Metadata(oldfile)

    # exif of resized image
    newExif = GExiv2.Metadata(newfile)

    # Figure out dimensions
    imgresize = Image.open(newfile)

    # save all exif data of orinal image to resized
    for tag in exif.get_exif_tags():
        newExif[tag] = exif[tag]

    # edit exif data - size 
    newExif['Exif.Photo.PixelXDimension'] = str(imgresize.size[0])
    newExif['Exif.Photo.PixelYDimension'] = str(imgresize.size[1])
    # FIXME: Doesn't work with PENTAX JPG
    # Error is: gi._glib.GError: Unsupported data area offset type
    newExif.save_file()

def resize_compute_width_height(fullfile,_megapixels):
    """Given image file and desired megapixels,
    computes the new width and height"""

    img = Image.open(fullfile)
    width,height=img.size

    current_megapixels=width*height/(2.0**20)
    scale=sqrt(_megapixels/float(current_megapixels))

    logger.debug('A resize scale would be %f'%(scale))
    # Can't make bigger, return false
    if scale>= 1.0:
        return None,None

    new_width=int(width*scale)
    new_height=int(height*scale)

    return new_width,new_height


def getexif_location(directory,fn):
    """
    directory - Dir where file is located
    fn - filename to check for EXIF GPS

    Returns touple of lat,lon if EXIF
    eg. (34.035460,-118.227885)
    files contains GPS info, otherwise returns
    None,None
    """
    lat=None
    lon=None

    sign_lat=+1.0
    sign_lon=+1.0
    # Check if photo as geo info already
    exif_tags=exifread.process_file(\
            open(os.path.join(directory,fn),'rb'))
    try:
        d,m,s=exif_tags['GPS GPSLongitude'].values
        # West is negative longitudes, change sign
        if exif_tags['GPS GPSLongitudeRef'].values=='W':
            sign_lon=-1.0
        lon=float(d.num) +float(m.num)/60.0 +float(s.num/float(s.den))/3600.0
        lon=lon*sign_lon
        d,m,s=exif_tags['GPS GPSLatitude'].values
        # South is negative latitude, change sign
        if exif_tags['GPS GPSLatitudeRef'].values=='S':
            sign_lat=-1.0
        lat=float(d.num)\
                +float(m.num)/60.0\
                +float(s.num/float(s.den))/3600.0
        lat=lat*sign_lat
    except:
        logger.debug("%s - Couldn't extract GPS info"%(fn))

    return lat,lon



import service_flickr
import service_facebook
import service_wp

class servicemanager:

    def __init__(self):
        self.services=[]
        self.services.append(service_flickr.service_flickr())
        self.services.append(service_wp.service_wp())
        self.services.append(service_facebook.service_facebook())

    def GetServices(self,filename):
        """Returns a list of service objects handling this file type"""
        objlist=[]
        for sobj in self.services:
            if sobj.KnowsFile(filename) :
                objlist.append(sobj)

        if len(objlist)==0:
            return None

        return objlist

    def GetServiceObj(self,servicename):
        """Given a service name string, returns
        the object that corresponds to the service"""
        for sobj in self.services:
            if sobj.GetName().lower()==servicename.lower():
                return sobj

        return None

    def PrintServices(self):
        for sobj in self.services:
            print sobj.GetName()



#!/usr/bin/python2

# This code logs us into facebook once, obtained from:
# http://blog.carduner.net/2010/05/

import os.path
import json
import urllib2
import urllib
import urlparse
import BaseHTTPServer
import webbrowser
 
APP_ID = '401958849934497'
APP_SECRET = '7b84ba01b576c1ff128c130d11b658d2'
ENDPOINT = 'graph.facebook.com'
REDIRECT_URI = 'http://localhost:8080/'
ACCESS_TOKEN = None
TOKEN_FILE = os.path.join(os.environ['HOME'],'.fb_access_token')

STATUS_TEMPLATE = u"{name}\033[0m: {message}"
 
def get_url(path, args=None):
    args = args or {}
    if ACCESS_TOKEN:
        args['access_token'] = ACCESS_TOKEN
    if 'access_token' in args or 'client_secret' in args:
        endpoint = "https://"+ENDPOINT
    else:
        endpoint = "http://"+ENDPOINT
    return endpoint+path+'?'+urllib.urlencode(args)
 
def get(path, args=None):
    return urllib2.urlopen(get_url(path, args=args)).read()
 
class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
    def do_GET(self):
        global ACCESS_TOKEN
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
 
        code = urlparse.parse_qs(urlparse.urlparse(self.path).query).get('code')
        code = code[0] if code else None
        if code is None:
            self.wfile.write("Sorry, authentication failed.")
            sys.exit(1)
        response = get('/oauth/access_token', {'client_id':APP_ID,
                                               'redirect_uri':REDIRECT_URI,
                                               'client_secret':APP_SECRET,
                                               'code':code})
        ACCESS_TOKEN = urlparse.parse_qs(response)['access_token'][0]
        open(TOKEN_FILE,'w').write(ACCESS_TOKEN)
        self.wfile.write("You have successfully logged in to facebook. "
                         "You can close this window now.")
 
def get_fb_token():
    """Returns access token, optionally launches browser session 
    to get the token via a facebook login.
    esterhui: got this code from the interweb, but can't find the reference now.
    """
    global ACCESS_TOKEN
    if not os.path.exists(TOKEN_FILE):
        print "Logging you in to facebook..."
        webbrowser.open(get_url('/oauth/authorize',
                                {'client_id':APP_ID,
                                 'redirect_uri':REDIRECT_URI,
                                 'scope':'manage_pages,publish_stream,photo_upload,user_photos,video_upload'}))
 
        httpd = BaseHTTPServer.HTTPServer(('127.0.0.1', 8080), RequestHandler)
        while ACCESS_TOKEN is None:
            httpd.handle_request()
    else:
        ACCESS_TOKEN = open(TOKEN_FILE).read()

    return ACCESS_TOKEN



#!/usr/bin/env python2
#
# Displays user's flickr sets on google maps with links back to flickr set.
#
# Example: http://gps.pythion.com
#
# Script to equery user flickr sets and build javascript variable
# to be loaded by google maps.


import sys
import argparse
import os
import logging
import random

import service_flickr
import json

logger=logging.getLogger('build_json_from_flickr')


# If lat/lon are the same, sprinkles
# around by this much so we do not get
# overlapping markers
POS_MOVE_METERS = 5000
POS_MOVE_DEG = POS_MOVE_METERS/111000.0

parser=argparse.ArgumentParser(description='Build JSON data from flickr sets to be loaded by google maps')
parser.add_argument("outputfile", help="File to write JSON data to")

def main():
    logger.setLevel(logging.ERROR)
    args=parser.parse_args()

    # Get our handle to flickr service
    flickr=service_flickr.service_flickr()

    # Get list of data sets and primary photo ids
    dsets=flickr._getphotosets()

    # Sort according to title, which will
    # always be yyyy-mm-dd - Name for my photosets
    keys=dsets.keys()
    keys.sort(reverse=True)

    data={}
    data['number_sets'] = len(keys)
    data['sets']=[]
    # Loop through each set
    for key in keys:
        d={}
        pid=dsets[key]['photo_id']
        lat,lon,acc=flickr._getphoto_location(pid)
        file_url=flickr._getphoto_url(pid)
        dinfo=flickr._getphoto_information(pid)
        print('%s : %s,%s\n  url: %s\npurl: %s\n'\
                %(key,lat,lon,dsets[key]['url'],file_url))

        d['photo_url']=dsets[key]['url'] # Link to set
        d['photo_file_url']=file_url # Link to actual file
        d['latitude']=lat
        d['longitude']=lon
        d['photo_id']=pid
        d['photo_title']=dinfo['title']
        d['set_title']=key
        data['sets'].append(d)

    # move lat/lon if they're too close to each other
    # so google can draw them separately
    redistribute_duplicates(data)

    # Save to database
    json.dump(data,open(args.outputfile,'w'))
    

def redistribute_duplicates(data):
    """Given a dictionary of photo sets, will look at lat/lon between
    sets, if they match, randomly move them around so the google map
    markeres do not overlap
    """
    
    coordinate_list=[]
    # Build a list of coordinates
    for myset in data['sets']:
        coordinate_list.append((myset['latitude'],myset['longitude']))

    for myset in data['sets']:
        lat=myset['latitude']
        lon=myset['longitude']
        item=(lat,lon)
        if coordinate_list.count(item) > 1:
            print("moving %s"%(myset['set_title']))
            random_number=random.random()
            myset['latitude']=str(random_number*POS_MOVE_DEG\
                    +float(myset['latitude']))
            myset['longitude']=str(random_number*POS_MOVE_DEG\
                    +float(myset['longitude']))

if __name__ == '__main__':
    main()



import sys
import os
import logging
import tempfile
import json 

from wordpress_xmlrpc import Client, WordPressPost
from wordpress_xmlrpc.methods.posts import GetPosts, NewPost, EditPost, DeletePost
from wordpress_xmlrpc.methods.users import GetUserInfo

logger=logging.getLogger('pusher')

# Where the wordpress login file sits, should be dictionary like
# this:
#
# ~/.pusher_wordpress
#
#{
#    "url" : "http://www.mydomain.com/xmlrpc.php",
#    "username" : "myusername",
#    "password" : "mypassword"
#}

#
WP_LOGIN_FILE = os.path.join(os.environ['HOME'],'.pusher_wordpress')

class service_wp:
    """Wordpress interface to easily push stuff to a wordpress blag
    via xmlrpc. A wordpress file ends on .wp and looks like this:
    title: My title [required]
    tags: south africa, pretoria, more tags [optional]
    category: travel,2012 [optional]
    --- <=== 3 dashes indicate separation of metadata and content
    Body stuff goes here
    """


    def __init__(self):
        self.name="wordpress"
        self.WP_EXTENTIONS=['.wp']
        self.DB_FILE=".wordpressdb"
        self.connected_to_wp=False
        self.WP_META_KEYS=['title','tags','category']
        self.META_SEPARATOR='---'

    def _connectToWP(self):
        """Establish the actual TCP connection to Flickr"""

        if self.connected_to_wp:
            logger.debug("Already connected to wp")
            return True

        # Load config from file
        info=json.load(open(WP_LOGIN_FILE,'r'))
        self.wp = Client(info['url'],\
                info['username'],\
                info['password'])

        logger.debug("Connecting to wp")

        self.connected_to_wp=True

        return True

    def GetName(self):
        return self.name

    def _isWPFile(self, filename):
        ext=os.path.splitext(filename)[1].lower()
        if ext in self.WP_EXTENTIONS:
            return True
        return False

    def KnowsFile(self,filename):
        """Looks at extension and decides if it knows
        how to manage this file"""
        if self._isWPFile(filename):
            return True
        return False

    def Upload(self,directory,filename):
        """Uploads/Updates/Replaces files"""

        db = self._loadDB(directory)

        logger.debug("wp: Attempting upload of %s"%(filename))

        # See if this already exists in our DB
        if db.has_key(filename):
            pid=db[filename]
            logger.debug('wp: Found %s in DB with post id %s'%(filename,pid))
        else:
            pid=None

        fullfile=os.path.join(directory,filename)

        fid=open(fullfile,'r');

        # Read meta data and content into dictionary
        post=self._readMetaAndContent(fid)

        #Connect to WP
        self._connectToWP()

        # If no pid, it means post is fresh off the press
        # and not uploaded yet!
        if not pid:
            # Get a PID by uploading
            pid=self.wp.call(NewPost(post))
            if pid:
                logger.debug("wp: Uploaded post with pid %s",pid)
                db[filename]=pid
                self._saveDB(directory,db)
                return True
            else:
                logger.error("wp: Couldn't upload post")
                return False
        else:
            # Already has PID, replace post
            logger.debug("wp: Replacing post with pid %s",pid)
            #FIXME: Check return value?!
            self.wp.call(EditPost(pid,post))
            return True

        return False

    def _readMetaAndContent(self,fid):
        """Reads meta data and content from file into WordPressPost() class.
        Returns the class

        If error, returns None
        """
        found_meta_separator=False

        d={}
        # Read lines until we find the meta separator
        while found_meta_separator is False:
            line = fid.readline()
            # Did we find the --- separator it?
            if line[0:len(self.META_SEPARATOR)]==self.META_SEPARATOR:
                found_meta_separator=True
            else:
                key=line.split(':')[0].strip().lower()
                if key in self.WP_META_KEYS:
                    d[key]=line.split(':')[1].strip()
                else:
                    logger.error("wp: Token '%s' not in list of known tokens %s"\
                            %(key,self.WP_META_KEYS))
                    return None

        if not d.has_key('title'):
            print("wp: A title: keyword is required!")

        d['content']=fid.readlines()
        d['content']=''.join(d['content'])

        
        # Let's transfer over to a wordpress post class
        post = WordPressPost()
        post.title=d['title']
        post.content=d['content']
        post.post_status='publish'

        post.terms_names={}
        if d.has_key('tags'):
            post.terms_names['post_tag']=d['tags'].split(',')
        if d.has_key('category'):
            post.terms_names['category']=d['category'].split(',')

        return post

    def Remove(self,directory,filename):
        """Deletes post from wordpress"""

        db = self._loadDB(directory)

        logger.debug("wp: Attempting to remove %s from wp"%(filename))

        # See if this already exists in our DB
        if db.has_key(filename):
            pid=db[filename]
            logger.debug('wp: Found %s in DB with post id %s'%(filename,pid))
        else:
            print("wp: %s not in our local DB file [%s]"\
                    %(filename,self.DB_FILE))
            return False

        self._connectToWP()
        self.wp.call(DeletePost(pid))

        del db[filename]
        self._saveDB(directory,db)

        return True

    def _loadDB(self,directory):
        # See if we can open our DB file
        logger.debug("Trying to open wp DB")
        try:
            db=json.load(open(os.path.join(directory,self.DB_FILE)))
        except IOError:
            db={}

        return db

    def _saveDB(self,directory,db):
        # See if we can open our DB file
        logger.debug("Trying to save wp DB")
        try:
            db=json.dump(db,open(os.path.join(directory,self.DB_FILE),'w'))
        except IOError:
            logger.warning("WP DB could not be saved")



#!/usr/bin/env python2
#
# Given GPS csv file (as saved by android gps logger app), builds
# a JSON object we can use to load into google maps

import sys
import argparse
import os
import logging

from dateutil import parser as dateparser
from pygeocoder import Geocoder

import json
import csv
import time
import glob
from itertools import chain

# Only lookup geo location if new point differs from 
# previous point by this many meters

POS_THRESHOLD_METERS = 1000

# Each degree is roughly 111km, here we convert from
# threshold in meters to degrees
POS_THRESHOLD_DEG = POS_THRESHOLD_METERS/111000.0
logger=logging.getLogger('build_json_from_gps')

parser=argparse.ArgumentParser(description='Build JSON data from gps CSV file to be loaded by google maps')
parser.add_argument("inputfile", help="File containing GPS data")
parser.add_argument("outputfile", help="File to write JSON data to")

def main():
    logger.setLevel(logging.INFO)
    logging.basicConfig()
    args=parser.parse_args()

    files=glob.glob(args.inputfile)
    
    print files

    # Sort input files
    file_list=[]
    for fn in files:
        file_list.append(fn)

    file_list.sort()

    lpos=[]
    lcache=[]
    for fn in file_list:
        pos,cache=lookupfile(fn)
        lpos.append(pos)
        lcache.append(cache)

    # Combine into one
    lpos=list(chain.from_iterable(lpos))
    lcache=list(chain.from_iterable(lcache))

    # Grab the last item only
    d1=lpos[-1]
    d2=lcache[-1]

    d_merge=dict(d1.items() + d2.items())

    d={}
    d['gps']=[d_merge]

    json.dump(d,open(args.outputfile,'w'))

    


def lookupfile(filename):
    """Returns dictionary of file content as well 
    as google reverse GEO data"""
    logger.info('Looking up %s'%(filename))

    # First open cache file to see if we already looked up this stuff
    dirname=os.path.dirname(filename)
    basefilename=os.path.basename(filename)

    CACHE_FILE = os.path.join(dirname,'.'+basefilename+'.gpscache')
    cache=loadCacheFile(CACHE_FILE)

    # Get the input file
    positions=parsePositionFile(filename)

    # If load didn't work, read again and lookup
    if not cache:
        logger.info("%s - No cache file found, looking up location"\
                %(basefilename))
        cache=lookupGeoInfo(positions)
        # Save to DB
        json.dump(cache,open(CACHE_FILE,'w'))
    else:
        logger.info("%s - Found cache file for locations"\
                %(basefilename))

    return positions,cache

def loadCacheFile(filename):
    """Tries to load the cache file where
    we store our reverse lookup info
    """

    try:
        d=json.load(open(filename,'r'))
    except:
        return None

    return d

def lookup_by_latlon(lat,lon):
    """Looks up location given lat,lon
    returns dictionary with address:
    d['street number']  : 1925
    d['route']          : Illinois Dr
    d['city']           : South Pasadena
    d['admin area 2']   : Los Angeles County
    d['admin area 1']   : California
    d['country']        : USA
    d['postal_code']    : 91030
    d['fullname']       : Full address string
    """

    logger.info("Looking up %s/%s via google"%(lat,lon))
    results = Geocoder.reverse_geocode(float(lat),float(lon))
    addrs=results.data[0]['address_components']
    d={}
    d['fullname']=str(results)
    for addr in addrs:
        if addr['types'][0]=='locality':
            d['city']=addr['short_name']
        elif addr['types'][0]=='street_number':
            d['street number']=addr['short_name']
        elif addr['types'][0]=='route':
            d['route']=addr['short_name']
        elif addr['types'][0]=='admin area 2':
            d['admin area 2']=addr['short_name']
        elif addr['types'][0]=='admin area 1':
            d['admin area 1']=addr['short_name']
        elif addr['types'][0]=='country':
            d['country']=addr['long_name']
        elif addr['types'][0]=='postal_code':
            d['postal_code']=addr['short_name']

    # Slow us down so google doesn't complain!
    time.sleep(0.1)

    return d

def parsePositionFile(filename):
    """
    Parses Android GPS logger csv file and returns list of dictionaries
    """
    l=[]
    with open( filename, "rb" ) as theFile:
        reader = csv.DictReader( theFile )
        for line in reader:
            # Convert the time string to something
            # a bit more human readable
            mytime=dateparser.parse(line['time'])
            line['strtime']=mytime.strftime("%d %b %Y, %H:%M UTC")
            l.append(line)
    return l

def lookupGeoInfo(positions):
    """Looks up lat/lon info with goole given a list
    of positions as parsed by parsePositionFile.
    Returns google results in form of dicionary
    """
    list_data=[]
    oldlat=0
    oldlon=0
    d={}
    for pos in positions:
        # Only lookup point if it is above threshold
        diff_lat=abs(float(pos['lat'])-oldlat)
        diff_lon=abs(float(pos['lon'])-oldlon)
        if (diff_lat>POS_THRESHOLD_DEG) or\
           (diff_lon>POS_THRESHOLD_DEG):
            d=lookup_by_latlon(pos['lat'],pos['lon'])
            oldlat=float(pos['lat'])
            oldlon=float(pos['lon'])
        else:
            logger.debug("Skipping %s/%s, close to prev"%(pos['lat'],pos['lon']))

        # Use fresh lookup value or old value
        list_data.append(d)

    logger.info('looked up %d positions'%(len(list_data)))
    return list_data

if __name__ == '__main__':
    main()




from pypu.pushercli import VERSION
from distutils.core import setup

setup(
    name='pypu',
    version=VERSION,
    packages=['pypu',],
    scripts=["bin/pypu"],
    author='Stephan Esterhuizen',
    author_email='esterhui@gmail.com',
    url='http://github.com/esterhui/pypu',
    license='GPLv2',
    long_description=open('README.rst').read(),
)



#!/usr/bin/env python2
import os
import argparse

try:
    import pypu.pusher as pusher
except ImportError:
    import pusher

import logging
import glob

VERSION='0.1.9'

parser=argparse.ArgumentParser(description='Upload stuff to flickr and wordpress (%s)'%(VERSION),version=VERSION)
parser.add_argument("action", help="Action flag: st,pull,push,add,rm")
parser.add_argument('files', metavar='file', type=str, nargs='+',
    help="Directory or file list to operate on")
parser.add_argument('-vv','--verbose',help="Verbose debug output",\
        action="store_true")
parser.add_argument('-d','--debug',help="Very Verbose debug output",\
        action="store_true")
parser.add_argument('-f','--force',help="Attempts to force operation",\
        action="store_true")
parser.add_argument('-s',help="Action only applies to given service (by default all services attempt to apply action). Use '<progname> services print' to print all supported services",\
        action="store",dest="service")

logger=logging.getLogger('pusher')

def main():
    status = pusher.status()
    # ---- Setup logging stuff ----
    logger.setLevel(logging.ERROR)
    args=parser.parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)
    if args.debug:
        logger.setLevel(logging.DEBUG)

    # By default service=None means all services,
    # unless user specified a service
    service=None
    if args.service:
        service=args.service

    if os.path.isdir(args.files[0]):
        directory=args.files[0]
        files=None
    else:
        directory=os.path.dirname(args.files[0])
        if not directory:
            directory='.'
        files=[]
        for fn in args.files:
            files.append(os.path.basename(fn))

    if args.action=='st' or args.action=='status':
        logger.debug("Directory is %s" %(args.files[0]))
        status.PrintStatus(directory,files,service)
    elif args.action=='add':
        if files is None:
            print "add: Must specify files to add"
            return 1
        status.UpdateStatus(directory,files,status.ST_ADDED,service)
    elif args.action=='push':
        status.UpdateStatus(directory,files,status.ST_UPTODATE,service)
    elif args.action=='rm':
        status.UpdateStatus(directory,files,status.ST_DELETED,service)
    elif args.action=='services':
        status.sman.PrintServices()
        

if __name__ == '__main__':
    main()
